package xyz.matj1.bp;

import mcdr.objects.impl.ObjectMotionWord;
import mcdr.objects.utils.RankedSortedCollectionDistHashcode;
import mcdr.sequence.SequenceMotionWords;
import mcdr.sequence.impl.SequenceMotionWordsDTW;
import mcdr.test.utils.ObjectCategoryMgmt.Category;
import mcdr.test.utils.ObjectMgmt;
import messif.objects.LocalAbstractObject;
import messif.objects.util.RankedAbstractObject;
import messif.operations.AnswerType;
import messif.operations.query.KNNQueryOperation;

import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * The tasks originally assigned to me so I would learn how to work with the provided data and libraries
 */
public class BasicTasks {
	public static void tasks(ObjectMgmt mwSequenceMgmt) {
		// evaluate distance=dissimilarity of two objects
		Iterator<LocalAbstractObject> mwSequenceIt = mwSequenceMgmt.getObjects().iterator();
		LocalAbstractObject mwSequence1 = mwSequenceIt.next();
		LocalAbstractObject mwSequence2 = mwSequenceIt.next();
		float distance = mwSequence1.getDistance(mwSequence2);
		System.out.println("Distance between " + Utils.sequenceToString(mwSequence1) + " and " + Utils.sequenceToString(mwSequence2)+" is "+distance);
		System.out.println("The category of object " + Utils.sequenceToString(mwSequence1) + " is " + mwSequenceMgmt.parseObjectCategoryId(mwSequence1));

		// evaluate kNN search - you probably won't be using this but it can be useful for some cross-checking
		LocalAbstractObject query = mwSequence1;
		Integer queryCategorySize = mwSequenceMgmt.getObjectCountInCategory(mwSequenceMgmt.getObjectCategories(query).get(0));
		KNNQueryOperation op = new KNNQueryOperation(query, queryCategorySize-1, false, AnswerType.ORIGINAL_OBJECTS, new RankedSortedCollectionDistHashcode());
		for (LocalAbstractObject dataObject : mwSequenceMgmt.getObjects()) {
			SequenceMotionWordsDTW mwObject = (SequenceMotionWordsDTW) dataObject;
			if (!mwObject.getLocatorURI().equals(query.getLocatorURI())) {
				op.addToAnswer(mwObject, query.getDistance(mwObject), null);
			}
		}
		System.out.println("Top "+(queryCategorySize-1)+" results for query " + Utils.sequenceToString(query)+": ");
		Iterator<RankedAbstractObject> answerIt = op.getAnswer();
		while (answerIt.hasNext()) {
			RankedAbstractObject resultObject = answerIt.next();
			System.out.println("  " + Utils.sequenceToString((SequenceMotionWords<ObjectMotionWord>)resultObject.getObject())+" - distance "+resultObject.getDistance());
		}


		// first task: evaluate kNN search for all objects with k=size of the query object's category
		// compute average precision and compare to results in ECIR paper - it should be the same or at least very similar

		Instant firstTaskTimer = Instant.now();

		Collection<Double> knnPrecisions = new ArrayList<>();
		for (LocalAbstractObject queryObject : mwSequenceMgmt.getObjects()) {
			Category queryObjectCategory = mwSequenceMgmt.getObjectCategories(queryObject).get(0);
			int queryObjectCategorySize = mwSequenceMgmt.getObjectCountInCategory(queryObjectCategory);

			KNNQueryOperation operation = new KNNQueryOperation(
				queryObject,
				queryObjectCategorySize - 1,
				false,
				AnswerType.ORIGINAL_OBJECTS,
				new RankedSortedCollectionDistHashcode()
			);
			for (LocalAbstractObject dataObject : mwSequenceMgmt.getObjects()) {
				if (!dataObject.getLocatorURI().equals(queryObject.getLocatorURI())) {
					operation.addToAnswer(dataObject, queryObject.getDistance(dataObject), null);
				}
			}

			Iterator<RankedAbstractObject> answer = operation.getAnswer();
			int countOfObjectsInTheSameCategory = 0;
			while (answer.hasNext()) {
				RankedAbstractObject resultObject = answer.next();
				if (mwSequenceMgmt.getObjectCategories(
					(LocalAbstractObject) resultObject.getObject()
				).get(0) == queryObjectCategory) {
					countOfObjectsInTheSameCategory++;
				}
			}

			double precision = (double)countOfObjectsInTheSameCategory / operation.getK();
			if(Double.isFinite(precision)) {
				knnPrecisions.add(precision);
			}
			/* Pokud objekt je sám ve své kategorii, k v KNN je 0
			a tehdy nemá smysl počítat precision. */
		}

		double averagePrecision = Utils.average(knnPrecisions);
		System.out.println();
		System.out.println("The first task: average precision: " + averagePrecision);
		System.out.println("time: " + Duration.between(firstTaskTimer, Instant.now()).toMillis() + " ms");

		// b task: for each category, compute the average distance to all categories
		// moje oprava: vypsat vzdálenosti každých dvou kategorií

		Instant secondTaskTimer = Instant.now();

		System.out.println();
		System.out.println("The second task:");

		Set<Entry<Category, List<LocalAbstractObject>>> categoryObjectsEntrySet = mwSequenceMgmt.getCategoryObjects().entrySet();
		for (Entry<Category, List<LocalAbstractObject>> entry1 : categoryObjectsEntrySet) {
			for (Entry<Category, List<LocalAbstractObject>> entry2 : categoryObjectsEntrySet) {
				if (entry1.getKey().id.compareTo(entry2.getKey().id) >= 0) {
					continue;
				}

				System.out.printf(
					"  %s, %s: %4f%n",
					entry1.getKey().id,
					entry2.getKey().id,
					Utils.distanceOfCategories(entry1.getValue(), entry2.getValue())
				);
			}

		}

		System.out.println("time: " + Duration.between(secondTaskTimer, Instant.now()).toMillis() + " ms");

		// third task: find the range of sizes of objects in individual categories

		Instant thirdTaskTimer = Instant.now();

		System.out.println();
		System.out.println("The third task:");

		for (Map.Entry<Category, List<LocalAbstractObject>> entry : mwSequenceMgmt.getCategoryObjects().entrySet()) {
			int minSize = Integer.MAX_VALUE;
			int maxSize = Integer.MIN_VALUE;

			List<Integer> sequenceLengths = new ArrayList<>();
			for (LocalAbstractObject item : entry.getValue()) {
				String[] uriArray = item.getLocatorURI().split("_");
				int length = Integer.parseInt(uriArray[uriArray.length - 1]);
				sequenceLengths.add(length);
			}

			for (int size : sequenceLengths) {
				if (size < minSize) {
					minSize = size;
				}
				if (size > maxSize) {
					maxSize = size;
				}
			}

			System.out.printf(
				"  %s: min: %d, max: %d%n",
				entry.getKey().id,
				minSize,
				maxSize
			);
		}

		System.out.println("time: " + Duration.between(thirdTaskTimer, Instant.now()).toMillis() + " ms");

	}
}
