package xyz.matj1.bp.types;

import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiFunction;
import java.util.function.BiPredicate;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.stream.Collector;
import java.util.stream.Collector.Characteristics;
import java.util.stream.Collectors;

public class MyMap<K, V> extends ConcurrentHashMap<K, V> {
	public MyMap() {
		super();
	}
	public MyMap(int initialCapacity) {
		super(initialCapacity);
	}
	public MyMap(int initialCapacity, float loadFactor) {
		super(initialCapacity, loadFactor);
	}
	public MyMap(int initialCapacity, float loadFactor, int concurrencyLevel) {
		super(initialCapacity, loadFactor, concurrencyLevel);
	}
	public MyMap(Map<? extends K, ? extends V> originalMap) {
		super(originalMap);
	}

	public static <K, V> MyMap<K, V> of(Map<? extends K, ? extends V> originalMap) {
		return new MyMap<>(originalMap);
	}

	public static <K, V> MyMap<K, V> ofEntries(Collection<Entry<? extends K, ? extends V>> entries) {
		return MyMap.of(entries.stream().collect(
			Collectors.toUnmodifiableMap(Entry::getKey, Entry::getValue)
		));
	}

	public static <K, V> Collector<K, MyMap<K, V>, MyMap<K, V>> keyCollector(Function<K, V> valueMapper) {
		return Collector.of(
			MyMap::new,
			(accumulator, key) -> accumulator.put(key, valueMapper.apply(key)),
			(accumulator, other) -> {
				accumulator.putAll(other);
				return accumulator;
			},
			Characteristics.CONCURRENT, Characteristics.IDENTITY_FINISH, Characteristics.UNORDERED
		);
	}

	public <K2, V2> MyMap<K2, V2> map(
		Function<? super K, ? extends K2> keyMapper,
		Function<? super V,  ? extends V2> valueMapper,
		BinaryOperator<V2> valueMerger
	) {
		var result = new MyMap<K2, V2>();

		this.entrySet().stream().parallel().forEach(entry -> {
			var mapKey = keyMapper.apply(entry.getKey());
			var mapValue = valueMapper.apply(entry.getValue());

			if (result.containsKey(mapKey)) {
				mapValue = valueMerger.apply(result.get(mapKey), mapValue);
			}

			result.put(mapKey, mapValue);
		});

		return result;
	}

	public <K2, V2> MyMap<K2, V2> map(
		Function<? super K, ? extends K2> keyMapper,
		Function<? super V,  ? extends V2> valueMapper
	) {
		return this.map(keyMapper, valueMapper, (a, b) -> b);
	}

	public <K2, V2> MyMap<K2, V2> map(
		BiFunction<? super K, ? super V, ? extends K2> keyMapper,
		BiFunction<? super K, ? super V,  ? extends V2> valueMapper,
		BinaryOperator<V2> valueMerger
	) {
		var result = new MyMap<K2, V2>();

		this.entrySet().stream().parallel().forEach(entry -> {
			var mapKey = keyMapper.apply(entry.getKey(), entry.getValue());
			var mapValue = valueMapper.apply(entry.getKey(), entry.getValue());

			if (result.containsKey(mapKey)) {
				mapValue = valueMerger.apply(result.get(mapKey), mapValue);
			}

			result.put(mapKey, mapValue);
		});

		return result;
	}

	public <K2, V2> MyMap<K2, V2> map(
		BiFunction<? super K, ? super V, ? extends K2> keyMapper,
		BiFunction<? super K, ? super V,  ? extends V2> valueMapper
	) {
		return this.map(keyMapper, valueMapper, (a, b) -> b);
	}

	public <V2> MyMap<K, V2> mapValues(Function<? super V,  ? extends V2> valueMapper) {
		var result = new MyMap<K, V2>();
		this.entrySet().stream().parallel().forEach(
			entry -> result.put(entry.getKey(), valueMapper.apply(entry.getValue()))
		);
		return result;
	}

	public MyMap<K, V> filter(BiPredicate<K, V> predicate) {
		var result = new MyMap<K, V>();

		this.entrySet().stream().parallel().forEach(entry -> {
			if (predicate.test(entry.getKey(), entry.getValue())) {
				result.put(entry.getKey(), entry.getValue());
		}});

		return result;
	}
}
